import threading
from typing import List, Optional

from ..message import AIMessage, BasicMessage
from ..tool import Tool
from .llm_proxy import LLMProxy


class LLMManager:
    """Thread-safe LLM connection pool manager"""

    def __init__(self):
        self.client_pool = {}  # {thread_id: LLMProxy}
        self.config_cache = None  # Current configuration
        self._lock = threading.Lock()

    def initialize_from_config(self, config):
        """Initialize LLM manager from ConfigModel"""
        with self._lock:
            self.config_cache = {
                'model_name': config.model_name.value,
                'base_url': config.base_url.value,
                'api_key': config.api_key.value,
                'model_azure': config.model_azure.value,
                'max_tokens': config.max_tokens.value,
                'extra_header': config.extra_header.value,
                'extra_body': config.extra_body.value,
                'enable_thinking': config.enable_thinking.value,
                'api_version': config.api_version.value,
            }

    def get_client(self) -> LLMProxy:
        """Get LLM client for current thread"""
        thread_id = threading.get_ident()

        if thread_id not in self.client_pool:
            if not self.config_cache:
                raise RuntimeError('LLMManager not initialized. Call initialize_from_config() first.')

            # Create new client for this thread
            self.client_pool[thread_id] = LLMProxy(
                model_name=self.config_cache['model_name'],
                base_url=self.config_cache['base_url'],
                api_key=self.config_cache['api_key'],
                model_azure=self.config_cache['model_azure'],
                max_tokens=self.config_cache['max_tokens'],
                extra_header=self.config_cache['extra_header'],
                extra_body=self.config_cache['extra_body'],
                enable_thinking=self.config_cache['enable_thinking'],
                api_version=self.config_cache['api_version'],
            )

        return self.client_pool[thread_id]

    async def call(
        self,
        msgs: List[BasicMessage],
        tools: Optional[List[Tool]] = None,
        show_status: bool = True,
        status_text: Optional[str] = None,
        use_streaming: bool = True,
        timeout: float = 20.0,
        interrupt_check: Optional[callable] = None,
        show_result: bool = True,
    ) -> AIMessage:
        """Unified LLM call interface"""
        client = self.get_client()
        return await client.call(
            msgs,
            tools,
            show_status=show_status,
            use_streaming=use_streaming,
            status_text=status_text,
            timeout=timeout,
            interrupt_check=interrupt_check,
            show_result=show_result,
        )

    async def cleanup_thread(self, thread_id: Optional[int] = None):
        """Clean up client for specific thread (or current thread)"""
        if thread_id is None:
            thread_id = threading.get_ident()

        if thread_id in self.client_pool:
            client = self.client_pool[thread_id]
            # Proactively close HTTP client connections
            try:
                if hasattr(client.client, 'client'):
                    http_client = client.client.client
                    if hasattr(http_client, 'aclose'):
                        await http_client.aclose()
            except Exception:
                # Ignore cleanup errors
                pass
            del self.client_pool[thread_id]

    def reset(self):
        """Reset all clients and configuration"""
        with self._lock:
            self.client_pool.clear()
            self.config_cache = None
